using System;
using System.Threading.Tasks;

using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Rendering;

namespace NSeguin.Dev.Web.HeroIcons;

public partial class HeroIcon : ComponentBase
{
    private Task<string>? _loadSvgTask;

    [Parameter]
    public HeroIconKind? Kind { get; set; }

    [Parameter]
    public HeroIconVariant Variant { get; set; }

    private string? RawSvg { get; set; }

    protected override void BuildRenderTree(RenderTreeBuilder builder)
    {
        builder.AddMarkupContent(0, RawSvg);
    }

    protected override void OnParametersSet()
    {
        if (Kind is not { } kind)
        {
            throw new InvalidOperationException("Kind cannot be null.");
        }

        var parameters = new HeroIconParameters(kind, Variant);
        _loadSvgTask = parameters.GetSvgAsync();
        if (_loadSvgTask.IsCompletedSuccessfully)
        {
            RawSvg = _loadSvgTask.Result;
        }
        else
        {
            _loadSvgTask.ContinueWith(
                task =>
                {
                    RawSvg = task.Result;
                    StateHasChanged();
                });
        }
    }

    protected override async Task OnInitializedAsync()
    {
        if (Kind is null)
        {
            throw new InvalidOperationException("Kind cannot be null.");
        }

        if (RawSvg is null && _loadSvgTask is not null)
        {
            await _loadSvgTask;
        }
    }
}
